using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ShapeBlending
{
    /// <summary>
    /// Fades between two colors.
    /// To start fading, call Start, and to stop
    /// fading, call Reset.
    /// </summary>
    class ColorFadeEffect
    {
        private Color m_startColor;
        private Color m_endColor;
        private Color m_currentColor;
        private int m_interval;

        private int m_direction = 0;
        private DateTime m_start;
        private float m_scale = 0.0f;

        public Color StartColor
        {
            get { return m_startColor; }
            set { m_startColor = value; }
        }

        public Color EndColor
        {
            get { return m_endColor; }
            set { m_endColor = value; }
        }

        public int Interval
        {
            get { return m_interval; }
            set { m_interval = value; }
        }

        public ColorFadeEffect(Color startColor, Color endColor, int interval)
        {
            m_startColor = startColor;
            m_endColor = endColor;
            m_currentColor = m_startColor;
            m_interval = interval;
        }

        public void Start()
        {
            if (m_direction != 1)
            {
                if (m_direction == -1)
                    m_start = DateTime.Now.Subtract(new TimeSpan((long)(TimeSpan.TicksPerMillisecond * (1 - m_scale) * m_interval)));
                else
                    m_start = DateTime.Now;
                m_direction = 1;
            }
        }

        public void Reset()
        {
            if (m_direction != -1)
            {
                m_start = DateTime.Now.Subtract(new TimeSpan((long)(TimeSpan.TicksPerMillisecond * (1-m_scale) * m_interval)));
                m_direction = -1;
            }
        }

        public Color GetColor()
        {
            if (m_direction == 0)
                return m_currentColor;

            m_scale = (float)(
                Math.Min(1, Math.Max(
                DateTime.Now.Subtract(m_start).TotalMilliseconds / m_interval, 0)));
            float scale = m_scale;
            if (m_direction == 1)
            {
                int a = (int)(m_startColor.A + ((float)m_endColor.A - m_startColor.A) * scale);
                int r = (int)(m_startColor.R + ((float)m_endColor.R - m_startColor.R) * scale);
                int g = (int)(m_startColor.G + ((float)m_endColor.G - m_startColor.G) * scale);
                int b = (int)(m_startColor.B + ((float)m_endColor.B - m_startColor.B) * scale);
                m_currentColor = Color.FromArgb(a, r, g, b);
            }
            else if (m_direction == -1)
            {
                int a = (int)(m_endColor.A + ((float)m_startColor.A - m_endColor.A) * scale);
                int r = (int)(m_endColor.R + ((float)m_startColor.R - m_endColor.R) * scale);
                int g = (int)(m_endColor.G + ((float)m_startColor.G - m_endColor.G) * scale);
                int b = (int)(m_endColor.B + ((float)m_startColor.B - m_endColor.B) * scale);
                m_currentColor = Color.FromArgb(a, r, g, b);
            }

            // Deactivate if reached target.
            if (scale == 1.0f)
                m_direction = 0;

            return m_currentColor;
        }
    }
}
