using System;
using System.Collections.Generic;
using System.Text;

namespace ShapeBlending
{
    class FrameTimer
    {
        private DateTime m_start;
        private DateTime m_lastCapture;
        private float m_deltaTime = 0;
        private int m_captures = 0;

        public float DeltaTime
        {
            get { return m_deltaTime; }
        }

        public float Time
        {
            get { return (float)m_lastCapture.Subtract(m_start).TotalSeconds; }
        }

        /// <summary>
        /// Gets the frames per second in average.
        /// </summary>
        public Single FramesPerSecond
        {
            get 
            {
                float t = Time;
                if (t == 0)
                    return 0;
                else
                    return (float)Math.Round((double)(m_captures / Time), 2); 
            }
        }

        /// <summary>
        /// Starts the frame timer.
        /// </summary>
        public void Start()
        {
            m_start = DateTime.Now;
            m_lastCapture = m_start;
            m_captures = 0;
        }

        public void Capture()
        {
            // Reset to start each 5 second.
            if (Time > 5)
            {
                m_start = m_lastCapture;
                m_captures = 0;
            }
            m_deltaTime = (float)DateTime.Now.Subtract(m_lastCapture).TotalSeconds;
            m_lastCapture = DateTime.Now;
            m_captures++;
        }
    }
}
