using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace ShapeBlending
{
    /// <summary>
    /// Contains materials, accessible by names or index.
    /// </summary>
    class MaterialManager
    {
        private Hashtable m_materials = new Hashtable();
        private ArrayList m_materialList = new ArrayList();

        /// <summary>
        /// Adds a new material.
        /// </summary>
        /// <param name="mat">The material to add.</param>
        /// <param name="name">The name of material.</param>
        public void AddMaterial(Material mat, string name)
        {
            m_materials.Add(name, mat);
            m_materialList.Add(mat);
        }

        /// <summary>
        /// Deletes a material.
        /// </summary>
        /// <param name="name">The name of material.</param>
        public void DeleteMaterial(string name)
        {
            Material mat = (Material)m_materials[name];
            m_materials.Remove(name);
            m_materialList.Remove(mat);
        }

        /// <summary>
        /// Gets the material by name.
        /// </summary>
        /// <param name="name">Name of material.</param>
        /// <returns>Returns the material name.</returns>
        public Material GetMaterial(string name)
        {
            return (Material)m_materials[name];
        }

        /// <summary>
        /// Gets the material by index.
        /// </summary>
        /// <param name="index">The index of material.</param>
        /// <returns>Returns the material found.</returns>
        public Material GetMaterial(int index)
        {
            if (index < 0)
                return null;
            return (Material)m_materialList[index];
        }

        /// <summary>
        /// Gets the index of a material by given name.
        /// </summary>
        /// <param name="name">The name of material.</param>
        /// <returns>Returns the index to material.</returns>
        public int IndexOf(string name)
        {
            Material mat = (Material)m_materials[name];
            return m_materialList.IndexOf(mat);
        }

        /// <summary>
        /// Gets the number of materials.
        /// </summary>
        public int Count
        {
            get { return m_materialList.Count; }
        }
    }
}
