using System;
using System.Collections.Generic;
using System.Text;

namespace ShapeBlending
{
    public class Matrix4f
    {
        float[] Entries = new float[16];

        public void LoadIdentity()
        {
            for (int i = 0; i < 16; i++)
                Entries[i] = 0.0f;
            Entries[0] = 1.0f;
            Entries[5] = 1.0f;
            Entries[10] = 1.0f;
            Entries[15] = 1.0f;
        }

        public void LoadZero()
        {
            for (int i = 0; i < 16; i++)
                Entries[i] = 0.0f;
        }

        public Matrix4f()
        {
            LoadIdentity();
        }

        public Matrix4f(float e0, float e1, float e2, float e3,
            float e4, float e5, float e6, float e7,
            float e8, float e9, float e10, float e11,
            float e12, float e13, float e14, float e15)
        {
            Entries[0] = e0;
            Entries[1] = e1;
            Entries[2] = e2;
            Entries[3] = e3;
            Entries[4] = e4;
            Entries[5] = e5;
            Entries[6] = e6;
            Entries[7] = e7;
            Entries[8] = e8;
            Entries[9] = e9;
            Entries[10] = e10;
            Entries[11] = e11;
            Entries[12] = e12;
            Entries[13] = e13;
            Entries[14] = e14;
            Entries[15] = e15;
        }

        public Matrix4f(float[] rhs)
        {
            rhs.CopyTo(Entries, 0);
        }

        public Matrix4f(Matrix4f rhs)
        {
            rhs.Entries.CopyTo(Entries, 0);
        }

        public void SetEntry(int position, float value)
        {
            if (position >= 0 && position <= 15)
                Entries[position] = value;
        }

        public float GetEntry(int position)
        {
            if (position >= 0 && position <= 15)
                return Entries[position];
            return 0;
        }

        public Vector4f GetRow(int position)
        {
            if (position == 0)
                return new Vector4f(Entries[0], Entries[4], Entries[8], Entries[12]);

            if (position == 1)
                return new Vector4f(Entries[1], Entries[5], Entries[9], Entries[13]);

            if (position == 2)
                return new Vector4f(Entries[2], Entries[6], Entries[10], Entries[14]);

            if (position == 3)
                return new Vector4f(Entries[3], Entries[7], Entries[11], Entries[15]);

            return new Vector4f(0.0f, 0.0f, 0.0f, 0.0f);
        }

        public Vector4f GetColumn(int position)
        {
            if (position == 0)
                return new Vector4f(Entries[0], Entries[1], Entries[2], Entries[3]);

            if (position == 1)
                return new Vector4f(Entries[4], Entries[5], Entries[6], Entries[7]);

            if (position == 2)
                return new Vector4f(Entries[8], Entries[9], Entries[10], Entries[11]);

            if (position == 3)
                return new Vector4f(Entries[12], Entries[13], Entries[14], Entries[15]);

            return new Vector4f(0.0f, 0.0f, 0.0f, 0.0f);
        }

        public static Matrix4f operator +(Matrix4f m, Matrix4f rhs)
        {
            return new Matrix4f(m.Entries[0] + rhs.Entries[0],
                        m.Entries[1] + rhs.Entries[1],
                        m.Entries[2] + rhs.Entries[2],
                        m.Entries[3] + rhs.Entries[3],
                        m.Entries[4] + rhs.Entries[4],
                        m.Entries[5] + rhs.Entries[5],
                        m.Entries[6] + rhs.Entries[6],
                        m.Entries[7] + rhs.Entries[7],
                        m.Entries[8] + rhs.Entries[8],
                        m.Entries[9] + rhs.Entries[9],
                        m.Entries[10] + rhs.Entries[10],
                        m.Entries[11] + rhs.Entries[11],
                        m.Entries[12] + rhs.Entries[12],
                        m.Entries[13] + rhs.Entries[13],
                        m.Entries[14] + rhs.Entries[14],
                        m.Entries[15] + rhs.Entries[15]);
        }

        public static Matrix4f operator -(Matrix4f m, Matrix4f rhs)
        {
            return new Matrix4f(m.Entries[0] - rhs.Entries[0],
                        m.Entries[1] - rhs.Entries[1],
                        m.Entries[2] - rhs.Entries[2],
                        m.Entries[3] - rhs.Entries[3],
                        m.Entries[4] - rhs.Entries[4],
                        m.Entries[5] - rhs.Entries[5],
                        m.Entries[6] - rhs.Entries[6],
                        m.Entries[7] - rhs.Entries[7],
                        m.Entries[8] - rhs.Entries[8],
                        m.Entries[9] - rhs.Entries[9],
                        m.Entries[10] - rhs.Entries[10],
                        m.Entries[11] - rhs.Entries[11],
                        m.Entries[12] - rhs.Entries[12],
                        m.Entries[13] - rhs.Entries[13],
                        m.Entries[14] - rhs.Entries[14],
                        m.Entries[15] - rhs.Entries[15]);
        }

        public static Matrix4f operator *(Matrix4f m, Matrix4f rhs)
        {
            //Optimise for matrices in which bottom row is (0, 0, 0, 1) in both matrices
            if (m.Entries[3] == 0.0f && m.Entries[7] == 0.0f && m.Entries[11] == 0.0f && m.Entries[15] == 1.0f &&
                rhs.Entries[3] == 0.0f && rhs.Entries[7] == 0.0f &&
                rhs.Entries[11] == 0.0f && rhs.Entries[15] == 1.0f)
            {
                return new Matrix4f(m.Entries[0] * rhs.Entries[0] + m.Entries[4] * rhs.Entries[1] + m.Entries[8] * rhs.Entries[2],
                                    m.Entries[1] * rhs.Entries[0] + m.Entries[5] * rhs.Entries[1] + m.Entries[9] * rhs.Entries[2],
                                    m.Entries[2] * rhs.Entries[0] + m.Entries[6] * rhs.Entries[1] + m.Entries[10] * rhs.Entries[2],
                                    0.0f,
                                    m.Entries[0] * rhs.Entries[4] + m.Entries[4] * rhs.Entries[5] + m.Entries[8] * rhs.Entries[6],
                                    m.Entries[1] * rhs.Entries[4] + m.Entries[5] * rhs.Entries[5] + m.Entries[9] * rhs.Entries[6],
                                    m.Entries[2] * rhs.Entries[4] + m.Entries[6] * rhs.Entries[5] + m.Entries[10] * rhs.Entries[6],
                                    0.0f,
                                    m.Entries[0] * rhs.Entries[8] + m.Entries[4] * rhs.Entries[9] + m.Entries[8] * rhs.Entries[10],
                                    m.Entries[1] * rhs.Entries[8] + m.Entries[5] * rhs.Entries[9] + m.Entries[9] * rhs.Entries[10],
                                    m.Entries[2] * rhs.Entries[8] + m.Entries[6] * rhs.Entries[9] + m.Entries[10] * rhs.Entries[10],
                                    0.0f,
                                    m.Entries[0] * rhs.Entries[12] + m.Entries[4] * rhs.Entries[13] + m.Entries[8] * rhs.Entries[14] + m.Entries[12],
                                    m.Entries[1] * rhs.Entries[12] + m.Entries[5] * rhs.Entries[13] + m.Entries[9] * rhs.Entries[14] + m.Entries[13],
                                    m.Entries[2] * rhs.Entries[12] + m.Entries[6] * rhs.Entries[13] + m.Entries[10] * rhs.Entries[14] + m.Entries[14],
                                    1.0f);
            }

            //Optimise for when bottom row of 1st matrix is (0, 0, 0, 1)
            if (m.Entries[3] == 0.0f && m.Entries[7] == 0.0f && m.Entries[11] == 0.0f && m.Entries[15] == 1.0f)
            {
                return new Matrix4f(m.Entries[0] * rhs.Entries[0] + m.Entries[4] * rhs.Entries[1] + m.Entries[8] * rhs.Entries[2] + m.Entries[12] * rhs.Entries[3],
                                    m.Entries[1] * rhs.Entries[0] + m.Entries[5] * rhs.Entries[1] + m.Entries[9] * rhs.Entries[2] + m.Entries[13] * rhs.Entries[3],
                                    m.Entries[2] * rhs.Entries[0] + m.Entries[6] * rhs.Entries[1] + m.Entries[10] * rhs.Entries[2] + m.Entries[14] * rhs.Entries[3],
                                    rhs.Entries[3],
                                    m.Entries[0] * rhs.Entries[4] + m.Entries[4] * rhs.Entries[5] + m.Entries[8] * rhs.Entries[6] + m.Entries[12] * rhs.Entries[7],
                                    m.Entries[1] * rhs.Entries[4] + m.Entries[5] * rhs.Entries[5] + m.Entries[9] * rhs.Entries[6] + m.Entries[13] * rhs.Entries[7],
                                    m.Entries[2] * rhs.Entries[4] + m.Entries[6] * rhs.Entries[5] + m.Entries[10] * rhs.Entries[6] + m.Entries[14] * rhs.Entries[7],
                                    rhs.Entries[7],
                                    m.Entries[0] * rhs.Entries[8] + m.Entries[4] * rhs.Entries[9] + m.Entries[8] * rhs.Entries[10] + m.Entries[12] * rhs.Entries[11],
                                    m.Entries[1] * rhs.Entries[8] + m.Entries[5] * rhs.Entries[9] + m.Entries[9] * rhs.Entries[10] + m.Entries[13] * rhs.Entries[11],
                                    m.Entries[2] * rhs.Entries[8] + m.Entries[6] * rhs.Entries[9] + m.Entries[10] * rhs.Entries[10] + m.Entries[14] * rhs.Entries[11],
                                    rhs.Entries[11],
                                    m.Entries[0] * rhs.Entries[12] + m.Entries[4] * rhs.Entries[13] + m.Entries[8] * rhs.Entries[14] + m.Entries[12] * rhs.Entries[15],
                                    m.Entries[1] * rhs.Entries[12] + m.Entries[5] * rhs.Entries[13] + m.Entries[9] * rhs.Entries[14] + m.Entries[13] * rhs.Entries[15],
                                    m.Entries[2] * rhs.Entries[12] + m.Entries[6] * rhs.Entries[13] + m.Entries[10] * rhs.Entries[14] + m.Entries[14] * rhs.Entries[15],
                                    rhs.Entries[15]);
            }

            //Optimise for when bottom row of 2nd matrix is (0, 0, 0, 1)
            if (rhs.Entries[3] == 0.0f && rhs.Entries[7] == 0.0f &&
                rhs.Entries[11] == 0.0f && rhs.Entries[15] == 1.0f)
            {
                return new Matrix4f(m.Entries[0] * rhs.Entries[0] + m.Entries[4] * rhs.Entries[1] + m.Entries[8] * rhs.Entries[2],
                                    m.Entries[1] * rhs.Entries[0] + m.Entries[5] * rhs.Entries[1] + m.Entries[9] * rhs.Entries[2],
                                    m.Entries[2] * rhs.Entries[0] + m.Entries[6] * rhs.Entries[1] + m.Entries[10] * rhs.Entries[2],
                                    m.Entries[3] * rhs.Entries[0] + m.Entries[7] * rhs.Entries[1] + m.Entries[11] * rhs.Entries[2],
                                    m.Entries[0] * rhs.Entries[4] + m.Entries[4] * rhs.Entries[5] + m.Entries[8] * rhs.Entries[6],
                                    m.Entries[1] * rhs.Entries[4] + m.Entries[5] * rhs.Entries[5] + m.Entries[9] * rhs.Entries[6],
                                    m.Entries[2] * rhs.Entries[4] + m.Entries[6] * rhs.Entries[5] + m.Entries[10] * rhs.Entries[6],
                                    m.Entries[3] * rhs.Entries[4] + m.Entries[7] * rhs.Entries[5] + m.Entries[11] * rhs.Entries[6],
                                    m.Entries[0] * rhs.Entries[8] + m.Entries[4] * rhs.Entries[9] + m.Entries[8] * rhs.Entries[10],
                                    m.Entries[1] * rhs.Entries[8] + m.Entries[5] * rhs.Entries[9] + m.Entries[9] * rhs.Entries[10],
                                    m.Entries[2] * rhs.Entries[8] + m.Entries[6] * rhs.Entries[9] + m.Entries[10] * rhs.Entries[10],
                                    m.Entries[3] * rhs.Entries[8] + m.Entries[7] * rhs.Entries[9] + m.Entries[11] * rhs.Entries[10],
                                    m.Entries[0] * rhs.Entries[12] + m.Entries[4] * rhs.Entries[13] + m.Entries[8] * rhs.Entries[14] + m.Entries[12],
                                    m.Entries[1] * rhs.Entries[12] + m.Entries[5] * rhs.Entries[13] + m.Entries[9] * rhs.Entries[14] + m.Entries[13],
                                    m.Entries[2] * rhs.Entries[12] + m.Entries[6] * rhs.Entries[13] + m.Entries[10] * rhs.Entries[14] + m.Entries[14],
                                    m.Entries[3] * rhs.Entries[12] + m.Entries[7] * rhs.Entries[13] + m.Entries[11] * rhs.Entries[14] + m.Entries[15]);
            }

            return new Matrix4f(m.Entries[0] * rhs.Entries[0] + m.Entries[4] * rhs.Entries[1] + m.Entries[8] * rhs.Entries[2] + m.Entries[12] * rhs.Entries[3],
                                m.Entries[1] * rhs.Entries[0] + m.Entries[5] * rhs.Entries[1] + m.Entries[9] * rhs.Entries[2] + m.Entries[13] * rhs.Entries[3],
                                m.Entries[2] * rhs.Entries[0] + m.Entries[6] * rhs.Entries[1] + m.Entries[10] * rhs.Entries[2] + m.Entries[14] * rhs.Entries[3],
                                m.Entries[3] * rhs.Entries[0] + m.Entries[7] * rhs.Entries[1] + m.Entries[11] * rhs.Entries[2] + m.Entries[15] * rhs.Entries[3],
                                m.Entries[0] * rhs.Entries[4] + m.Entries[4] * rhs.Entries[5] + m.Entries[8] * rhs.Entries[6] + m.Entries[12] * rhs.Entries[7],
                                m.Entries[1] * rhs.Entries[4] + m.Entries[5] * rhs.Entries[5] + m.Entries[9] * rhs.Entries[6] + m.Entries[13] * rhs.Entries[7],
                                m.Entries[2] * rhs.Entries[4] + m.Entries[6] * rhs.Entries[5] + m.Entries[10] * rhs.Entries[6] + m.Entries[14] * rhs.Entries[7],
                                m.Entries[3] * rhs.Entries[4] + m.Entries[7] * rhs.Entries[5] + m.Entries[11] * rhs.Entries[6] + m.Entries[15] * rhs.Entries[7],
                                m.Entries[0] * rhs.Entries[8] + m.Entries[4] * rhs.Entries[9] + m.Entries[8] * rhs.Entries[10] + m.Entries[12] * rhs.Entries[11],
                                m.Entries[1] * rhs.Entries[8] + m.Entries[5] * rhs.Entries[9] + m.Entries[9] * rhs.Entries[10] + m.Entries[13] * rhs.Entries[11],
                                m.Entries[2] * rhs.Entries[8] + m.Entries[6] * rhs.Entries[9] + m.Entries[10] * rhs.Entries[10] + m.Entries[14] * rhs.Entries[11],
                                m.Entries[3] * rhs.Entries[8] + m.Entries[7] * rhs.Entries[9] + m.Entries[11] * rhs.Entries[10] + m.Entries[15] * rhs.Entries[11],
                                m.Entries[0] * rhs.Entries[12] + m.Entries[4] * rhs.Entries[13] + m.Entries[8] * rhs.Entries[14] + m.Entries[12] * rhs.Entries[15],
                                m.Entries[1] * rhs.Entries[12] + m.Entries[5] * rhs.Entries[13] + m.Entries[9] * rhs.Entries[14] + m.Entries[13] * rhs.Entries[15],
                                m.Entries[2] * rhs.Entries[12] + m.Entries[6] * rhs.Entries[13] + m.Entries[10] * rhs.Entries[14] + m.Entries[14] * rhs.Entries[15],
                                m.Entries[3] * rhs.Entries[12] + m.Entries[7] * rhs.Entries[13] + m.Entries[11] * rhs.Entries[14] + m.Entries[15] * rhs.Entries[15]);
        }

        public static Matrix4f operator *(Matrix4f m, float rhs)
        {
            return new Matrix4f(m.Entries[0] * rhs,
                        m.Entries[1] * rhs,
                        m.Entries[2] * rhs,
                        m.Entries[3] * rhs,
                        m.Entries[4] * rhs,
                        m.Entries[5] * rhs,
                        m.Entries[6] * rhs,
                        m.Entries[7] * rhs,
                        m.Entries[8] * rhs,
                        m.Entries[9] * rhs,
                        m.Entries[10] * rhs,
                        m.Entries[11] * rhs,
                        m.Entries[12] * rhs,
                        m.Entries[13] * rhs,
                        m.Entries[14] * rhs,
                        m.Entries[15] * rhs);
        }

        public static Matrix4f operator *(float scaleFactor, Matrix4f m)
        {
            return m * scaleFactor;
        }

        public new bool Equals(object o)
        {
            Matrix4f rhs = (Matrix4f)o;
            for (int i = 0; i < 16; i++)
            {
                if (Entries[i] != rhs.Entries[i])
                    return false;
            }
            return true;
        }

        public static Vector4f operator *(Matrix4f m, Vector4f rhs)
        {
            //Optimise for matrices in which bottom row is (0, 0, 0, 1)
            if (m.Entries[3] == 0.0f && m.Entries[7] == 0.0f && m.Entries[11] == 0.0f && m.Entries[15] == 1.0f)
            {
                return new Vector4f(m.Entries[0] * rhs.X
                            + m.Entries[4] * rhs.Y
                            + m.Entries[8] * rhs.Z
                            + m.Entries[12] * rhs.W,

                                m.Entries[1] * rhs.X
                            + m.Entries[5] * rhs.Y
                            + m.Entries[9] * rhs.Z
                            + m.Entries[13] * rhs.W,

                                m.Entries[2] * rhs.X
                            + m.Entries[6] * rhs.Y
                            + m.Entries[10] * rhs.Z
                            + m.Entries[14] * rhs.W,

                                rhs.W);
            }

            return new Vector4f(m.Entries[0] * rhs.X
                            + m.Entries[4] * rhs.Y
                            + m.Entries[8] * rhs.Z
                            + m.Entries[12] * rhs.W,

                                m.Entries[1] * rhs.X
                            + m.Entries[5] * rhs.Y
                            + m.Entries[9] * rhs.Z
                            + m.Entries[13] * rhs.W,

                                m.Entries[2] * rhs.X
                            + m.Entries[6] * rhs.Y
                            + m.Entries[10] * rhs.Z
                            + m.Entries[14] * rhs.W,

                                m.Entries[3] * rhs.X
                            + m.Entries[7] * rhs.Y
                            + m.Entries[11] * rhs.Z
                            + m.Entries[15] * rhs.W);
        }

        public Vector3f GetRotatedVector3f(Vector3f rhs)
        {
            return new Vector3f(Entries[0] * rhs.X + Entries[4] * rhs.Y + Entries[8] * rhs.Z,
                    Entries[1] * rhs.X + Entries[5] * rhs.Y + Entries[9] * rhs.Z,
                    Entries[2] * rhs.X + Entries[6] * rhs.Y + Entries[10] * rhs.Z);
        }

        public Vector3f GetInverseRotatedVector3f(Vector3f rhs)
        {
            //rotate by transpose:
            return new Vector3f(Entries[0] * rhs.X + Entries[1] * rhs.Y + Entries[2] * rhs.Z,
                            Entries[4] * rhs.X + Entries[5] * rhs.Y + Entries[6] * rhs.Z,
                            Entries[8] * rhs.X + Entries[9] * rhs.Y + Entries[10] * rhs.Z);
        }

        public Vector3f GetTranslatedVector3f(Vector3f rhs)
        {
            return new Vector3f(rhs.X + Entries[12], rhs.Y + Entries[13], rhs.Z + Entries[14]);
        }

        public Vector3f GetInverseTranslatedVector3f(Vector3f rhs)
        {
            return new Vector3f(rhs.X - Entries[12], rhs.Y - Entries[13], rhs.Z - Entries[14]);
        }

        public Matrix4f GetInverse()
        {
            Matrix4f result = GetInverseTranspose();
            result = result.GetTranspose();

            return result;
        }

        public Matrix4f GetTranspose()
        {
            return new Matrix4f(Entries[0], Entries[4], Entries[8], Entries[12],
                        Entries[1], Entries[5], Entries[9], Entries[13],
                        Entries[2], Entries[6], Entries[10], Entries[14],
                        Entries[3], Entries[7], Entries[11], Entries[15]);
        }

        public Matrix4f GetInverseTranspose()
        {
            Matrix4f result = new Matrix4f();

            float[] tmp = new float[12];												//temporary pair storage
            float det;													//determinant

            //calculate pairs for first 8 elements (cofactors)
            tmp[0] = Entries[10] * Entries[15];
            tmp[1] = Entries[11] * Entries[14];
            tmp[2] = Entries[9] * Entries[15];
            tmp[3] = Entries[11] * Entries[13];
            tmp[4] = Entries[9] * Entries[14];
            tmp[5] = Entries[10] * Entries[13];
            tmp[6] = Entries[8] * Entries[15];
            tmp[7] = Entries[11] * Entries[12];
            tmp[8] = Entries[8] * Entries[14];
            tmp[9] = Entries[10] * Entries[12];
            tmp[10] = Entries[8] * Entries[13];
            tmp[11] = Entries[9] * Entries[12];

            //calculate first 8 elements (cofactors)
            result.SetEntry(0, tmp[0] * Entries[5] + tmp[3] * Entries[6] + tmp[4] * Entries[7]
                            - tmp[1] * Entries[5] - tmp[2] * Entries[6] - tmp[5] * Entries[7]);

            result.SetEntry(1, tmp[1] * Entries[4] + tmp[6] * Entries[6] + tmp[9] * Entries[7]
                            - tmp[0] * Entries[4] - tmp[7] * Entries[6] - tmp[8] * Entries[7]);

            result.SetEntry(2, tmp[2] * Entries[4] + tmp[7] * Entries[5] + tmp[10] * Entries[7]
                            - tmp[3] * Entries[4] - tmp[6] * Entries[5] - tmp[11] * Entries[7]);

            result.SetEntry(3, tmp[5] * Entries[4] + tmp[8] * Entries[5] + tmp[11] * Entries[6]
                            - tmp[4] * Entries[4] - tmp[9] * Entries[5] - tmp[10] * Entries[6]);

            result.SetEntry(4, tmp[1] * Entries[1] + tmp[2] * Entries[2] + tmp[5] * Entries[3]
                            - tmp[0] * Entries[1] - tmp[3] * Entries[2] - tmp[4] * Entries[3]);

            result.SetEntry(5, tmp[0] * Entries[0] + tmp[7] * Entries[2] + tmp[8] * Entries[3]
                            - tmp[1] * Entries[0] - tmp[6] * Entries[2] - tmp[9] * Entries[3]);

            result.SetEntry(6, tmp[3] * Entries[0] + tmp[6] * Entries[1] + tmp[11] * Entries[3]
                            - tmp[2] * Entries[0] - tmp[7] * Entries[1] - tmp[10] * Entries[3]);

            result.SetEntry(7, tmp[4] * Entries[0] + tmp[9] * Entries[1] + tmp[10] * Entries[2]
                            - tmp[5] * Entries[0] - tmp[8] * Entries[1] - tmp[11] * Entries[2]);

            //calculate pairs for second 8 elements (cofactors)
            tmp[0] = Entries[2] * Entries[7];
            tmp[1] = Entries[3] * Entries[6];
            tmp[2] = Entries[1] * Entries[7];
            tmp[3] = Entries[3] * Entries[5];
            tmp[4] = Entries[1] * Entries[6];
            tmp[5] = Entries[2] * Entries[5];
            tmp[6] = Entries[0] * Entries[7];
            tmp[7] = Entries[3] * Entries[4];
            tmp[8] = Entries[0] * Entries[6];
            tmp[9] = Entries[2] * Entries[4];
            tmp[10] = Entries[0] * Entries[5];
            tmp[11] = Entries[1] * Entries[4];

            //calculate second 8 elements (cofactors)
            result.SetEntry(8, tmp[0] * Entries[13] + tmp[3] * Entries[14] + tmp[4] * Entries[15]
                            - tmp[1] * Entries[13] - tmp[2] * Entries[14] - tmp[5] * Entries[15]);

            result.SetEntry(9, tmp[1] * Entries[12] + tmp[6] * Entries[14] + tmp[9] * Entries[15]
                            - tmp[0] * Entries[12] - tmp[7] * Entries[14] - tmp[8] * Entries[15]);

            result.SetEntry(10, tmp[2] * Entries[12] + tmp[7] * Entries[13] + tmp[10] * Entries[15]
                            - tmp[3] * Entries[12] - tmp[6] * Entries[13] - tmp[11] * Entries[15]);

            result.SetEntry(11, tmp[5] * Entries[12] + tmp[8] * Entries[13] + tmp[11] * Entries[14]
                            - tmp[4] * Entries[12] - tmp[9] * Entries[13] - tmp[10] * Entries[14]);

            result.SetEntry(12, tmp[2] * Entries[10] + tmp[5] * Entries[11] + tmp[1] * Entries[9]
                            - tmp[4] * Entries[11] - tmp[0] * Entries[9] - tmp[3] * Entries[10]);

            result.SetEntry(13, tmp[8] * Entries[11] + tmp[0] * Entries[8] + tmp[7] * Entries[10]
                            - tmp[6] * Entries[10] - tmp[9] * Entries[11] - tmp[1] * Entries[8]);

            result.SetEntry(14, tmp[6] * Entries[9] + tmp[11] * Entries[11] + tmp[3] * Entries[8]
                            - tmp[10] * Entries[11] - tmp[2] * Entries[8] - tmp[7] * Entries[9]);

            result.SetEntry(15, tmp[10] * Entries[10] + tmp[4] * Entries[8] + tmp[9] * Entries[9]
                            - tmp[8] * Entries[9] - tmp[11] * Entries[10] - tmp[5] * Entries[8]);

            // calculate determinant
            det = Entries[0] * result.GetEntry(0)
                    + Entries[1] * result.GetEntry(1)
                    + Entries[2] * result.GetEntry(2)
                    + Entries[3] * result.GetEntry(3);

            if (det == 0.0f)
            {
                return null;
            }

            result = result / det;

            return result;
        }

        public static Matrix4f operator /(Matrix4f m, float rhs)
        {
            if (rhs == 0 || rhs == 1)
                return m;

            return m * (1 / rhs);
        }

        public Matrix4f GetAffineInvert()
        {
            //return the transpose of the rotation part
            //and the negative of the inverse rotated translation part
            return new Matrix4f(Entries[0],
                                Entries[4],
                                Entries[8],
                                0.0f,
                                Entries[1],
                                Entries[5],
                                Entries[9],
                                0.0f,
                                Entries[2],
                                Entries[6],
                                Entries[10],
                                0.0f,
                                -(Entries[0] * Entries[12] + Entries[1] * Entries[13] + Entries[2] * Entries[14]),
                                -(Entries[4] * Entries[12] + Entries[5] * Entries[13] + Entries[6] * Entries[14]),
                                -(Entries[8] * Entries[12] + Entries[9] * Entries[13] + Entries[10] * Entries[14]),
                                1.0f);
        }

        public Matrix4f GetAffineInverseTranspose()
        {
            //return the transpose of the rotation part
            //and the negative of the inverse rotated translation part
            //transposed
            return new Matrix4f(Entries[0],
                                Entries[1],
                                Entries[2],
                                -(Entries[0] * Entries[12] + Entries[1] * Entries[13] + Entries[2] * Entries[14]),
                                Entries[4],
                                Entries[5],
                                Entries[6],
                                -(Entries[4] * Entries[12] + Entries[5] * Entries[13] + Entries[6] * Entries[14]),
                                Entries[8],
                                Entries[9],
                                Entries[10],
                                -(Entries[8] * Entries[12] + Entries[9] * Entries[13] + Entries[10] * Entries[14]),
                                0.0f, 0.0f, 0.0f, 1.0f);
        }

        public void SetTranslation(Vector3f translation)
        {
            LoadIdentity();
            SetTranslationPart(translation);
        }

        public void SetScale(Vector3f scaleFactor)
        {
            LoadIdentity();

            Entries[0] = scaleFactor.X;
            Entries[5] = scaleFactor.Y;
            Entries[10] = scaleFactor.Z;
        }

        public void SetUniformScale(float scaleFactor)
        {
            Entries[0] = scaleFactor;
            Entries[5] = scaleFactor;
            Entries[10] = scaleFactor;
        }

        public void SetRotationAxis(double angle, Vector3f axis)
        {
            Vector3f u = axis.GetNormalized();

            float sinAngle = (float)Math.Sin(Math.PI * angle / 180);
            float cosAngle = (float)Math.Cos(Math.PI * angle / 180);
            float oneMinusCosAngle = 1.0f - cosAngle;

            LoadIdentity();

            Entries[0] = (u.X) * (u.X) + cosAngle * (1 - (u.X) * (u.X));
            Entries[4] = (u.X) * (u.Y) * (oneMinusCosAngle) - sinAngle * u.Z;
            Entries[8] = (u.X) * (u.Z) * (oneMinusCosAngle) + sinAngle * u.Y;

            Entries[1] = (u.X) * (u.Y) * (oneMinusCosAngle) + sinAngle * u.Z;
            Entries[5] = (u.Y) * (u.Y) + cosAngle * (1 - (u.Y) * (u.Y));
            Entries[9] = (u.Y) * (u.Z) * (oneMinusCosAngle) - sinAngle * u.X;

            Entries[2] = (u.X) * (u.Z) * (oneMinusCosAngle) - sinAngle * u.Y;
            Entries[6] = (u.Y) * (u.Z) * (oneMinusCosAngle) + sinAngle * u.X;
            Entries[10] = (u.Z) * (u.Z) + cosAngle * (1 - (u.Z) * (u.Z));
        }

        public void SetRotationX(double angle)
        {
            LoadIdentity();

            Entries[5] = (float)Math.Cos(Math.PI * angle / 180);
            Entries[6] = (float)Math.Sin(Math.PI * angle / 180);

            Entries[9] = -Entries[6];
            Entries[10] = Entries[5];
        }

        public void SetRotationY(double angle)
        {
            LoadIdentity();

            Entries[0] = (float)Math.Cos(Math.PI * angle / 180);
            Entries[2] = -(float)Math.Sin(Math.PI * angle / 180);

            Entries[8] = -Entries[2];
            Entries[10] = Entries[0];
        }

        public void SetRotationZ(double angle)
        {
            LoadIdentity();

            Entries[0] = (float)Math.Cos(Math.PI * angle / 180);
            Entries[1] = (float)Math.Sin(Math.PI * angle / 180);

            Entries[4] = -Entries[1];
            Entries[5] = Entries[0];
        }

        public void SetRotationEuler(double angleX, double angleY, double angleZ)
        {
            LoadIdentity();
            SetRotationPartEuler(angleX, angleY, angleZ);
        }


        void SetPerspective(float left, float right, float bottom,
                                        float top, float n, float f)
        {
            float nudge = 0.999f;		//prevent artifacts with infinite far plane

            LoadZero();

            //check for division by 0
            if (left == right || top == bottom || n == f)
                return;

            Entries[0] = (2 * n) / (right - left);

            Entries[5] = (2 * n) / (top - bottom);

            Entries[8] = (right + left) / (right - left);
            Entries[9] = (top + bottom) / (top - bottom);

            if (f != -1)
            {
                Entries[10] = -(f + n) / (f - n);
            }
            else		//if f==-1, use an infinite far plane
            {
                Entries[10] = -nudge;
            }

            Entries[11] = -1;

            if (f != -1)
            {
                Entries[14] = -(2 * f * n) / (f - n);
            }
            else		//if f==-1, use an infinite far plane
            {
                Entries[14] = -2 * n * nudge;
            }
        }

        void SetPerspective(float fovy, float aspect, float n, float f)
        {
            float left, right, top, bottom;

            //convert fov from degrees to radians
            fovy *= (float)Math.PI / 180;

            top = (float)(n * Math.Tan(fovy / 2.0f));
            bottom = -top;

            left = aspect * bottom;
            right = aspect * top;

            SetPerspective(left, right, bottom, top, n, f);
        }

        void SetOrtho(float left, float right, float bottom,
                                    float top, float n, float f)
        {
            LoadIdentity();

            Entries[0] = 2.0f / (right - left);

            Entries[5] = 2.0f / (top - bottom);

            Entries[10] = -2.0f / (f - n);

            Entries[12] = -(right + left) / (right - left);
            Entries[13] = -(top + bottom) / (top - bottom);
            Entries[14] = -(f + n) / (f - n);
        }

        public void SetTranslationPart(Vector3f translation)
        {
            Entries[12] = translation.X;
            Entries[13] = translation.Y;
            Entries[14] = translation.Z;
        }

        public void SetRotationPartEuler(double angleX, double angleY, double angleZ)
        {
            double cr = Math.Cos(Math.PI * angleX / 180);
            double sr = Math.Sin(Math.PI * angleX / 180);
            double cp = Math.Cos(Math.PI * angleY / 180);
            double sp = Math.Sin(Math.PI * angleY / 180);
            double cy = Math.Cos(Math.PI * angleZ / 180);
            double sy = Math.Sin(Math.PI * angleZ / 180);

            Entries[0] = (float)(cp * cy);
            Entries[1] = (float)(cp * sy);
            Entries[2] = (float)(-sp);

            double srsp = sr * sp;
            double crsp = cr * sp;

            Entries[4] = (float)(srsp * cy - cr * sy);
            Entries[5] = (float)(srsp * sy + cr * cy);
            Entries[6] = (float)(sr * cp);

            Entries[8] = (float)(crsp * cy + sr * sy);
            Entries[9] = (float)(crsp * sy - sr * cy);
            Entries[10] = (float)(cr * cp);
        }
    }
}
