using System;
using System.Collections.Generic;
using System.Text;
using Tao.OpenGl;

namespace ShapeBlending
{
    class Mesh
    {
        private float[][] m_vertices;
        private float[][] m_normals;
        private int[][] m_indices;
        private int[] m_materials;
        private MaterialManager m_materialManager;

        /// <summary>
        /// Gets or sets the material manager.
        /// </summary>
        public MaterialManager MaterialManager
        {
            get { return m_materialManager; }
            set { m_materialManager = value; }
        }

        /// <summary>
        /// Gets or sets the vertices.
        /// </summary>
        public float[][] Vertices
        {
            get { return m_vertices; }
            set { m_vertices = value; }
        }

        /// <summary>
        /// Gets or sets the normals.
        /// </summary>
        public float[][] Normals
        {
            get { return m_normals; }
            set { m_normals = value; }
        }

        /// <summary>
        /// Gets or sets the indices.
        /// </summary>
        public int[][] Indices
        {
            get { return m_indices; }
            set { m_indices = value; }
        }

        /// <summary>
        /// Gets or sets the materials.
        /// </summary>
        public int[] Materials
        {
            get { return m_materials; }
            set { m_materials = value; }
        }

        /// <summary>
        /// Draws the mesh.
        /// </summary>
        public void Draw()
        {
            int indiceCount = m_indices.GetLength(0);
            for (int i = 0; i < indiceCount; i++)
            {
                int[] polygon = (int[])m_indices[i];
                if (m_materials != null)
                {
                    Material mat = m_materialManager.GetMaterial(m_materials[i]);
                    if (mat != null)
                    {
                        Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat.Ambient);
                        Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat.Diffuse);
                        Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat.Specular);
                        Gl.glMaterialf(Gl.GL_FRONT, Gl.GL_SHININESS, mat.Shineness);
                    }
                }

                Gl.glBegin(Gl.GL_POLYGON);
                for (int j = 0; j < polygon.Length; j += 3)
                {
                    float[] vertex = (float[])m_vertices[(int)polygon[j]];
                    float[] normal = (float[])m_normals[(int)polygon[j + 2]];
                    Gl.glNormal3fv(normal);
                    Gl.glVertex3fv(vertex);
                }
                Gl.glEnd();
            }
        }
    }
}
