using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ShapeBlending
{
    class MtlReader
    {
        public static MaterialManager ReadMaterialLib(string file)
        {
            MaterialManager manager = new MaterialManager();
            StreamReader r = new StreamReader(file);

            // Change the number decimal separator to 
            // be compatible with obj format.
            System.Globalization.CultureInfo oldCulture = Application.CurrentCulture;
            System.Globalization.CultureInfo culture = (System.Globalization.CultureInfo)Application.CurrentCulture.Clone();
            culture.NumberFormat.NumberDecimalSeparator = ".";
            Application.CurrentCulture = culture;
            Material currentMaterial = null;
            string currentName = null;
            float red, green, blue, alpha;

            while (!r.EndOfStream)
            {
                string nextLine = r.ReadLine();
                // Ignore comments.
                if (nextLine.StartsWith("#"))
                    continue;
                string[] values = nextLine.Split(' ');
                if (nextLine.StartsWith("newmtl "))
                {
                    // Add old material to material manager.
                    if (currentMaterial != null)
                        manager.AddMaterial(currentMaterial, currentName);
                    // Start on new material.
                    currentName = values[1];
                    currentMaterial = new Material();
                }
                else if (nextLine.StartsWith("Kd "))
                {
                    red = float.Parse(values[1]);
                    green = float.Parse(values[2]);
                    blue = float.Parse(values[3]);
                    currentMaterial.Diffuse = new float[] { red, green, blue };
                }
                else if (nextLine.StartsWith("Ka "))
                {
                    red = float.Parse(values[1]);
                    green = float.Parse(values[2]);
                    blue = float.Parse(values[3]);
                    currentMaterial.Ambient = new float[] { red, green, blue };
                }
                else if (nextLine.StartsWith("Ks "))
                {
                    red = float.Parse(values[1]);
                    green = float.Parse(values[2]);
                    blue = float.Parse(values[3]);
                    currentMaterial.Specular = new float[] { red, green, blue };
                }
                else if (nextLine.StartsWith("d ") || nextLine.StartsWith("Tr "))
                {
                    alpha = float.Parse(values[1]);
                    currentMaterial.Alpha = alpha;
                }
            }
            r.Close();

            // Add last material.
            if (currentMaterial != null)
                manager.AddMaterial(currentMaterial, currentName);

            // Set decimal separator back.
            Application.CurrentCulture = oldCulture;

            return manager;
        }
    }
}
