using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Collections;
using System.Windows.Forms;

namespace ShapeBlending
{
    class ObjReader
    {
        public static Mesh Read(string file)
        {
            Mesh m = new Mesh();
            StreamReader r = new StreamReader(file);

            ArrayList vertices = new ArrayList();
            ArrayList normals = new ArrayList();
            ArrayList indices = new ArrayList();
            ArrayList materials = new ArrayList();
            int currentMaterial = -1;

            // Change the number decimal separator to 
            // be compatible with obj format.
            System.Globalization.CultureInfo oldCulture = Application.CurrentCulture;
            System.Globalization.CultureInfo culture = (System.Globalization.CultureInfo)Application.CurrentCulture.Clone();
            culture.NumberFormat.NumberDecimalSeparator = ".";
            Application.CurrentCulture = culture;

            while (!r.EndOfStream)
            {
                string nextLine = r.ReadLine();
                // Ignore comments.
                if (nextLine.StartsWith("#"))
                    continue;
                string[] values = nextLine.Split(' ');
                if (nextLine.StartsWith("v "))
                {
                    // Read vertex.
                    float x = float.Parse(values[1]);
                    float y = float.Parse(values[2]);
                    float z = float.Parse(values[3]);
                    vertices.Add(new float[] { x, y, z });
                }
                else if (nextLine.StartsWith("vn "))
                {
                    // Read normal.
                    float x = float.Parse(values[1]);
                    float y = float.Parse(values[2]);
                    float z = float.Parse(values[3]);
                    normals.Add(new float[] { x, y, z });
                }
                else if (nextLine.StartsWith("f "))
                { 
                    // Read indices.
                    ArrayList indiList = new ArrayList();
                    for (int i = 1; i < values.Length; i++)
                    {
                        string[] indiceValues = values[i].Split('/');
                        int i1 = int.Parse(indiceValues[0]) - 1;
                        int i2 = int.Parse(indiceValues[1]) - 1;
                        int i3 = int.Parse(indiceValues[2]) - 1;
                        indiList.AddRange(new int[] { i1, i2, i3 });
                    }
                    int[] indi = new int[indiList.Count];
                    indiList.CopyTo(indi, 0);
                    indices.Add(indi);
                    // Add the current material to polygon.
                    materials.Add(currentMaterial);
                }
                else if (nextLine.StartsWith("mtllib "))
                {
                    // Read material lib.
                    string materialFile = values[1];
                    materialFile = Path.GetDirectoryName(file) + "\\" + materialFile;
                    MaterialManager materialManager = MtlReader.ReadMaterialLib(materialFile);
                    m.MaterialManager = materialManager;
                }
                else if (nextLine.StartsWith("usemtl "))
                {
                    // Use material.
                    string name = values[1];
                    currentMaterial = m.MaterialManager.IndexOf(name);
                }
            }
            r.Close();

            // Set indices.
            int[][] meshIndices = new int[indices.Count][];
            indices.CopyTo(meshIndices, 0);
            m.Indices = meshIndices;

            // Set materials.
            int[] meshMaterials = new int[materials.Count];
            materials.CopyTo(meshMaterials, 0);
            m.Materials = meshMaterials;

            // Set mesh vertices.
            float[][] meshVertices = new float[vertices.Count][];
            vertices.CopyTo(meshVertices, 0);
            m.Vertices = meshVertices;

            // Set mesh normals.
            float[][] meshNormals = new float[normals.Count][];
            normals.CopyTo(meshNormals, 0);
            m.Normals = meshNormals;

            // Set decimal separator back.
            Application.CurrentCulture = oldCulture;

            return m;
        }
    }
}
