using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Tao.OpenGl;
using Tao.FreeGlut;
using System.Drawing;

namespace ShapeBlending
{
    class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            new Program();
        }

        private GuiLook m_look = new GuiLook();
        private ControlManager m_controlManager;
        private FrameTimer m_timer = new FrameTimer();

        private Mesh m_meshTest1;
        private Mesh m_meshTest2;
        private Mesh m_meshTest3;
        private Mesh m_meshTest4;
        private Mesh m_outMesh;

        private Mesh m_eyes;
        private Mesh m_hat;

        SimpleSlideBar m_slidebar1;
        SimpleSlideBar m_slidebar2;
        SimpleSlideBar m_slidebar3;

        SimpleButton m_closeMouthButton;
        SimpleButton m_smileButton;
        SimpleButton m_openMouthButton;

        SimpleLabel m_fpsLabel;

        CameraNavigation m_camera;

        public Program()
        {
            // Create new glut window.
            GlutWindow glutWindow = new GlutWindow();
            // Link glut window to control manager.
            m_controlManager = new ControlManager(glutWindow);
            glutWindow.Title = "Shape Blending";
            glutWindow.Display += new EventHandler(Display);
            glutWindow.LoadTextures += new EventHandler(LoadTextures);
            glutWindow.SetUpLights += new EventHandler(SetUpLights);
            glutWindow.KeyUp += new KeyEventHandler(KeyUp);

            m_camera = new CameraNavigation();

            m_timer.Start();

            glutWindow.Run();
        }

        public void LoadTextures(object sender, EventArgs e)
        {
            // Add slidebar 1.
            m_slidebar1 = new SimpleSlideBar(m_look);
            m_slidebar1.Position = new Point(40, 40);
            m_slidebar1.Size = new Size(150, 30);
            m_controlManager.AddControl(m_slidebar1);
            m_slidebar1.ValueChanged += new EventHandler(SlideBarChanged);

            // Add slidebar 2.
            m_slidebar2 = new SimpleSlideBar(m_look);
            m_slidebar2.Position = new Point(40, 80);
            m_slidebar2.Size = m_slidebar1.Size;
            m_controlManager.AddControl(m_slidebar2);
            m_slidebar2.ValueChanged += new EventHandler(SlideBarChanged);

            // Add slidebar 3.
            m_slidebar3 = new SimpleSlideBar(m_look);
            m_slidebar3.Position = new Point(40, 120);
            m_slidebar3.Size = m_slidebar1.Size;
            m_controlManager.AddControl(m_slidebar3);
            m_slidebar3.ValueChanged += new EventHandler(SlideBarChanged);

            // Close mouth button.
            m_closeMouthButton = new SimpleButton(m_look);
            m_closeMouthButton.Position = new Point(40, 180);
            m_closeMouthButton.Size = new Size(150, 40);
            m_closeMouthButton.Text = "Close mouth";
            m_controlManager.AddControl(m_closeMouthButton);
            m_closeMouthButton.Clicked += new EventHandler(CloseMouthButton_Clicked);

            // Smile button.
            m_smileButton = new SimpleButton(m_look);
            m_smileButton.Position = new Point(40, 220);
            m_smileButton.Size = new Size(150, 40);
            m_smileButton.Text = "Smile";
            m_controlManager.AddControl(m_smileButton);
            m_smileButton.Clicked += new EventHandler(SmileButton_Clicked);

            // Open mouth button.
            m_openMouthButton = new SimpleButton(m_look);
            m_openMouthButton.Position = new Point(40, 260);
            m_openMouthButton.Size = m_smileButton.Size;
            m_openMouthButton.Text = "Open mouth";
            m_controlManager.AddControl(m_openMouthButton);
            m_openMouthButton.Clicked += new EventHandler(OpenMouthButton_Clicked);

            // Info label.
            SimpleLabel infoLabel = new SimpleLabel(m_look);
            infoLabel.Position = new Point(470, 370);
            infoLabel.Text = "Use W, A, S, D to control camera";
            infoLabel.Size = new Size(100, 50);
            m_controlManager.AddControl(infoLabel);

            // FPS label.
            m_fpsLabel = new SimpleLabel(m_look);
            m_fpsLabel.Position = new Point(470, 420);
            m_fpsLabel.Brush = Brushes.Lime;
            m_fpsLabel.Text = "FPS: ";
            m_fpsLabel.Size = new Size(100, 500);
            m_controlManager.AddControl(m_fpsLabel);

            m_meshTest1 = ObjReader.Read(Application.StartupPath + "\\testobj1.obj");
            m_meshTest2 = ObjReader.Read(Application.StartupPath + "\\testobj2.obj");
            m_meshTest3 = ObjReader.Read(Application.StartupPath + "\\testobj3.obj");
            m_meshTest4 = ObjReader.Read(Application.StartupPath + "\\testobj4.obj");
            m_eyes = ObjReader.Read(Application.StartupPath + "\\eyes.obj");
            m_hat = ObjReader.Read(Application.StartupPath + "\\hatt.obj");

            m_outMesh = new Mesh();
            m_outMesh.Indices = m_meshTest1.Indices;
            m_outMesh.MaterialManager = m_meshTest1.MaterialManager;
            m_outMesh.Materials = m_meshTest1.Materials;
            m_outMesh.Normals = m_meshTest1.Normals;
            m_outMesh.Vertices = m_meshTest1.Vertices;

            m_camera.Target = new Point3f(0.0f, 0.5f, 0.0f);
            m_camera.Position = new Point3f(0.0f, 1.0f, 4.0f);
            m_camera.UpVector = new Point3f(0.0f, 1.0f, 0.0f);
        }

        public void CloseMouthButton_Clicked(object sender, EventArgs e)
        { 
            // Close mouth.
            m_slidebar1.Value = 1.0f;
            m_slidebar2.Value = 0.0f;
            m_slidebar3.Value = 0.0f;
            UpdateVertices();
        }

        public void SmileButton_Clicked(object sender, EventArgs e)
        {
            // Smile.
            m_slidebar1.Value = 0.0f;
            m_slidebar2.Value = 1.0f;
            m_slidebar3.Value = 0.0f;
            UpdateVertices();
        }

        public void OpenMouthButton_Clicked(object sender, EventArgs e)
        { 
            // Open mouth.
            m_slidebar1.Value = 0.0f;
            m_slidebar2.Value = 0.0f;
            m_slidebar3.Value = 1.0f;
            UpdateVertices();
        }

        public void SlideBarChanged(object sender, EventArgs e)
        {
            UpdateVertices();
        }

        private void UpdateVertices()
        { 
            // Blend vertices.
            m_outMesh.Vertices = Blending.Blend(m_meshTest1.Vertices,
                new float[][][] { m_meshTest2.Vertices,
                m_meshTest3.Vertices,
                m_meshTest4.Vertices},
                new float[] { m_slidebar1.Value,
                m_slidebar2.Value,
                m_slidebar3.Value});
        }

        public void SetUpLights(object sender, EventArgs e)
        {
            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glEnable(Gl.GL_LIGHT0);
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, new float[] { 0.5f, 0.5f, 0.5f, 1.0f });
            Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, new float[] { 1.0f, 1.0f, 1.0f, 1.0f });
        }

        /// <summary>
        /// Displays the 3D graphics.
        /// </summary>
        public void Display(object sender, EventArgs e)
        {
            m_fpsLabel.Text = "FPS: " + m_timer.FramesPerSecond.ToString();
            m_timer.Capture();

            Gl.glClear(Gl.GL_COLOR_BUFFER_BIT |
                Gl.GL_DEPTH_BUFFER_BIT |
                Gl.GL_ACCUM_BUFFER_BIT);
            Gl.glMatrixMode(Gl.GL_MODELVIEW);

            // Hide back faces to make drawing faster.
            Gl.glEnable(Gl.GL_CULL_FACE);
            Gl.glCullFace(Gl.GL_BACK);

            Gl.glPushMatrix();
            Gl.glEnable(Gl.GL_LIGHTING);
            Gl.glShadeModel(Gl.GL_SMOOTH);

            // Let the camera navigation class
            // set up the camera transform.
            m_camera.SetUpCamera();

            m_outMesh.Draw();
            m_eyes.Draw();
            m_hat.Draw();

            Gl.glPopMatrix();

            Gl.glDisable(Gl.GL_CULL_FACE);
            Gl.glDisable(Gl.GL_LIGHTING);
            // Draw controls.
            m_controlManager.Draw();

            // End drawing.
            Gl.glFlush();
        }

        public void KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.W)
                // Rotate up.
                m_camera.RotateAround(new Point3f(1.0f, 0.0f, 0.0f), -5);
            else if (e.KeyCode == Keys.S)
                // Rotate down.
                m_camera.RotateAround(new Point3f(1.0f, 0.0f, 0.0f), 5);
            else if (e.KeyCode == Keys.A)
                // Rotate left.
                m_camera.RotateAround(new Point3f(0.0f, 1.0f, 0.0f), -5);
            else if (e.KeyCode == Keys.D)
                // Rotate right.
                m_camera.RotateAround(new Point3f(0.0f, 1.0f, 0.0f), 5);
        }
    }
}
