using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace ShapeBlending
{
    class SimpleButton : SimpleControl
    {
        private ColorFadeEffect m_fadeText;
        private ColorFadeEffect m_fadeBorder;
        private ColorFadeEffect m_fadeBackground;
        private string m_text = "SimpleButton";
        private GuiLook m_look;
        private GlutWindow m_window;

        public event EventHandler Clicked;

        private bool m_mouseDown = false;

        public string Text
        {
            get { return m_text; }
            set { m_text = value; }
        }

        /// <summary>
        /// Returns the position of button.
        /// </summary>
        public override Point Position
        {
            get
            {
                if (m_mouseDown)
                    return new Point(m_position.X + 2, m_position.Y + 1);
                else
                    return base.Position;
            }
            set
            {
                base.Position = value;
            }
        }

        public SimpleButton(GuiLook look)
        {
            m_look = look;
            m_fadeText = new ColorFadeEffect(
                m_look.InactiveTextColor,
                m_look.ActiveTextColor, m_look.FadeInterval);
            m_fadeBorder = new ColorFadeEffect(
                m_look.InactiveBorderColor,
                m_look.ActiveBorderColor, m_look.FadeInterval);
            m_fadeBackground = new ColorFadeEffect(
                m_look.InactiveControlColor,
                m_look.ActiveControlColor, m_look.FadeInterval);
            m_size = new Size(80, 50);
        }

        public const int ROUND_OFFSET = 10;

        public override void Draw(Graphics g)
        {
            int w = m_size.Width - 1;
            int h = m_size.Height - 1;
            // Draw button.
            Point p1 = new Point(ROUND_OFFSET, ROUND_OFFSET);
            Point p2 = new Point(w - ROUND_OFFSET, ROUND_OFFSET);
            Point p3 = new Point(ROUND_OFFSET, h - ROUND_OFFSET);
            Point p4 = new Point(w - ROUND_OFFSET, h - ROUND_OFFSET);

            Point p12 = new Point(w / 2, ROUND_OFFSET / 2);
            Point p24 = new Point(w, h / 2);
            Point p43 = new Point(w / 2, h - ROUND_OFFSET / 2);
            Point p31 = new Point(0, h / 2);

            Point[] ps =  new Point[] { p1, p12, p2, p24, p4, p43, p3, p31 };

            g.FillClosedCurve(new SolidBrush(m_fadeBackground.GetColor()), ps);
            g.DrawClosedCurve(new Pen(m_fadeBorder.GetColor(), 1.0f), ps);

            SizeF textSize = g.MeasureString(m_text, m_look.Font);
            
            g.DrawString(m_text, m_look.Font, new SolidBrush(m_fadeText.GetColor()), 
                m_size.Width / 2 - textSize.Width / 2, m_size.Height / 2 - textSize.Height / 2);
        }

        public override bool CaptureMouse(int x, int y, GlutWindow window)
        {
            // Capture mouse.
            m_fadeText.Start();
            m_fadeBorder.Start();
            m_fadeBackground.Start();
            m_window = window;
            m_window.MouseUp += new MouseEventHandler(MouseUp);
            m_window.MouseDown += new MouseEventHandler(MouseDown);
            return true;
        }

        public void MouseDown(object sender, MouseEventArgs e)
        {
            m_mouseDown = true;
        }

        public void MouseUp(object sender, MouseEventArgs e)
        {
            if (m_mouseDown)
                if (Clicked != null)
                {
                    // Raise clicked event.
                    Clicked(this, new EventArgs());
                }
            m_mouseDown = false;
        }

        public override bool ReleaseMouse(int x, int y)
        {
            // Release the mouse if outside control.
            if (!(x >= m_position.X && y >= m_position.Y &&
                x <= m_position.X + m_size.Width &&
                y <= m_position.Y + m_size.Height))
            {
                m_fadeText.Reset();
                m_fadeBorder.Reset();
                m_fadeBackground.Reset();
                m_window.MouseUp -= new MouseEventHandler(MouseUp);
                m_window.MouseDown -= new MouseEventHandler(MouseDown);
                m_window = null;
                return true;
            }
            return false;
        }
    }
}
  