using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ShapeBlending
{
    /// <summary>
    /// Base control for Simple controls.
    /// </summary>
    class SimpleControl
    {
        protected Point m_position;
        protected Size m_size;

        /// <summary>
        /// Gets or sets the size of position.
        /// </summary>
        public virtual Point Position
        {
            get { return m_position; }
            set { m_position = value; }
        }

        /// <summary>
        /// Gets or sets the size of control.
        /// </summary>
        public virtual Size Size
        {
            get { return m_size; }
            set { m_size = value; }
        }

        /// <summary>
        /// Tells whether control want's to use mouse
        /// to something.
        /// The control gets the focus until it has released
        /// the mouse.
        /// </summary>
        /// <param name="x">The x position of mouse.</param>
        /// <param name="y">The y position of mouse.</param>
        /// <returns>Returns true if capturing, false if not.</returns>
        public virtual bool CaptureMouse(int x, int y, GlutWindow window)
        {
            return false;
        }

        /// <summary>
        /// Releases mouse position.
        /// </summary>
        /// <param name="x">The x position of mouse.</param>
        /// <param name="y">The y position of mouse.</param>
        /// <returns></returns>
        public virtual bool ReleaseMouse(int x, int y)
        {
            return true;
        }

        public virtual void Draw(Graphics g)
        {
            // Don't draw anything.
        //    SolidBrush brush = new SolidBrush(Color.FromArgb(150, Color.Red));
        //    g.FillRectangle(brush, 0, 0, m_size.Width, m_size.Height);
        //    brush = new SolidBrush(Color.FromArgb(100, Color.Blue));
        //    g.FillEllipse(brush, 0, 0, m_size.Width, m_size.Height);
        //    g.DrawString("Hallo", new Font("Arial", 10), Brushes.Black, 10, 10);
        //
        }
    }
}
