using System;
using System.Collections.Generic;
using System.Text;

namespace ShapeBlending
{
    public class Vector4f
    {
        public float X;
        public float Y;
        public float Z;
        public float W;

        public Vector4f()
        {
            X = 0;
            Y = 0;
            Z = 0;
            W = 0;
        }

        public Vector4f(float newX, float newY, float newZ, float newW)
        {
            X = newX;
            Y = newY;
            Z = newZ;
            W = newW;
        }

        public Vector4f(float[] rhs)
        {
            X = rhs[0];
            Y = rhs[1];
            Z = rhs[2];
            W = rhs[3];
        }

        public Vector4f(Vector4f rhs)
        {
            X = rhs.X;
            Y = rhs.Y;
            Z = rhs.Z;
            W = rhs.W;
        }

        public Vector4f(Vector3f rhs)
        {
            X = rhs.X;
            Y = rhs.Y;
            Z = rhs.Z;
            W = 1.0f;
        }

        public void Set(float newX, float newY, float newZ, float newW)
        {
            X = newX;
            Y = newY;
            Z = newZ;
            W = newW;
        }

        public void LoadZero()
        {
            X = 0.0f;
            Y = 0.0f;
            Z = 0.0f;
            W = 0.0f;
        }

        public void LoadOne()
        {
            X = 1.0f;
            Y = 1.0f;
            Z = 1.0f;
            W = 1.0f;
        }

        public float DotProduct(Vector4f rhs)
        {
            return X * rhs.X + Y * rhs.Y + Z * rhs.Z + W * rhs.W;
        }

        public Vector4f GetRotatedX(double angle)
        {
            Vector3f v3d = new Vector3f(X, Y, Z);
            v3d = v3d.GetRotatedX(angle);
            return new Vector4f(v3d.X, v3d.Y, v3d.Z, W);
        }

        public Vector4f GetRotatedY(double angle)
        {
            Vector3f v3d = new Vector3f(X, Y, Z);
            v3d = v3d.GetRotatedY(angle);
            return new Vector4f(v3d.X, v3d.Y, v3d.Z, W);
        }

        public Vector4f GetRotatedZ(double angle)
        {
            Vector3f v3d = new Vector3f(X, Y, Z);
            v3d = v3d.GetRotatedZ(angle);
            return new Vector4f(v3d.X, v3d.Y, v3d.Z, W);
        }

        public Vector4f GetRotatedAxis(double angle, Vector3f axis)
        {
            Vector3f v3d = new Vector3f(X, Y, Z);
            v3d = v3d.GetRotatedAxis(angle, axis);
            return new Vector4f(v3d.X, v3d.Y, v3d.Z, W);
        }

        public Vector4f Lerp(Vector4f v2, float factor)
        {
            return this * (1.0f - factor) + v2 * factor;
        }

        public Vector4f QuadricInterpolate(Vector4f v2, Vector4f v3, float factor)
        {
            return this * (1.0f - factor) * (1.0f - factor) + 2 * v2 * factor * (1.0f - factor) + v3 * factor * factor;
        }

        public static Vector4f operator +(Vector4f v, Vector4f rhs)
        {
            return new Vector4f(v.X + rhs.X, v.Y + rhs.Y, v.Z + rhs.Z, v.W + rhs.W);
        }

        public static Vector4f operator -(Vector4f v, Vector4f rhs)
        {
            return new Vector4f(v.X - rhs.X, v.Y - rhs.Y, v.Z - rhs.Z, v.W - rhs.W);
        }

        public static Vector4f operator * (Vector4f v, float rhs)
        {
            return new Vector4f(v.X * rhs, v.Y * rhs, v.Z * rhs, v.W * rhs);
        }

        public static Vector4f operator /(Vector4f v, float rhs)
        { 
            return (rhs == 0 ? new Vector4f(0, 0, 0, 0) : 
                new Vector4f(v.X / rhs, v.Y / rhs, v.Z / rhs, v.W / rhs));
        }

        public static Vector4f operator *(float rhs, Vector4f v)
        {
            return v * rhs;
        }

        public new bool Equals(object o)
        {
            Vector4f rhs = (Vector4f)o;
            if (rhs.X == X && rhs.Y == Y && rhs.Z == Z && rhs.W == W)
                return true;
            return false;
        }
    }
}
